/*
 * Reveal | quickmag.js
 * Copyright 2005-2006 by Michael Wu
 * Released under GPL v2
 */

var QuickMag = {
	magOn: false,
	boxWidth: 200,
	mag_factor: 1.2,
	mag_count: 1,
	canvas: null,
	magbox: null,
	rx: 0,
	ry: 0,
	scaled_width: 0,
	ignore_event: false,

	QueryInterface: function (aIID)
	{
		if (aIID.equals(Components.interfaces.nsIPrefBranch2) ||
		    aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
		    aIID.equals(Components.interfaces.nsISupports))
			return this;

		throw Components.results.NS_NOINTERFACE;
	},

	observe: function (aSubject, aTopic, aData)
	{
		if (aTopic != "nsPref:changed")
			return;

		switch (aData) {
		case "reveal.quickmag":
			if (getBoolPref("reveal.quickmag"))
				getBrowser().mPanelContainer.addEventListener("click", this.toggleMag, false);
			else
				getBrowser().mPanelContainer.removeEventListener("click", this.toggleMag, false);
			break;
		case "reveal.quickmag_size":
			this.boxWidth = this.prefs.getIntPref("quickmag_size");
			this.canvas.width = this.canvas.height = this.boxWidth;
			this.scaled_width = Math.ceil(this.boxWidth/this.mag_factor) + 1;

			var ctx = this.canvas.getContext("2d");
			ctx.save();
			ctx.scale(this.mag_factor, this.mag_factor);
			break;
		}
	},

	calcMag: function (x, y) {
		var b = getBrowser().selectedBrowser;
		var doc = b.contentDocument;
		this.rx = this.ry = 0;

		x -= b.boxObject.x;
		y -= b.boxObject.y;

		if (doc.documentElement != null && doc.documentElement.scrollLeft != null) {
			this.rx = doc.documentElement.scrollLeft;
			this.ry = doc.documentElement.scrollTop;
		}

		if (this.rx == 0 && this.ry == 0 && doc.body != null) {
			this.rx = doc.body.scrollLeft;
			this.ry = doc.body.scrollTop;
		}

		this.rx += x;
		this.ry += y;
	},

	renderMag: function () {
		var ctx = this.canvas.getContext("2d");

		ctx.drawWindow(getBrowser().selectedBrowser.contentWindow, this.rx - (this.scaled_width/2), this.ry - (this.scaled_width/2), this.scaled_width, this.scaled_width, "rgb(0,0,0)");
	},

	adjustZoom: function (e) {
		if (e.detail > 0 && QuickMag.mag_count <= 8) {
			QuickMag.mag_factor *= 1.3;
			QuickMag.mag_count += 1;
		} else if (e.detail < 0 && QuickMag.mag_count > 1) {
			QuickMag.mag_factor /= 1.3;
			QuickMag.mag_count -= 1;
		}

		QuickMag.scaled_width = Math.ceil(QuickMag.boxWidth/QuickMag.mag_factor) + 1;
		var ctx = QuickMag.canvas.getContext("2d");
		ctx.restore();
		ctx.save();
		ctx.scale(QuickMag.mag_factor, QuickMag.mag_factor);
		QuickMag.renderMag();
	},

	trackMouse: function (e) {
		if (!(e.target instanceof HTMLCanvasElement))
			return;

		var half_width = QuickMag.boxWidth/2;
		var boxObj = e.currentTarget.boxObject;
		if (QuickMag.ignore_event) {
			QuickMag.ignore_event = false;
			return;
		}
		QuickMag.magbox.style.left = (e.clientX - half_width) + "px";
		QuickMag.magbox.style.top = (e.clientY - half_width) + "px";
		QuickMag.calcMag(e.clientX, e.clientY);
		QuickMag.renderMag();
	},

	disableMag: function () {
		if (!QuickMag.magOn)
			return;

		var mwindow = document.getElementById("main-window");
		QuickMag.magOn = false;
		QuickMag.magbox.hidden = true;
		window.removeEventListener("mousemove", QuickMag.trackMouse, false);
		mwindow.removeChild(QuickMag.magbox);
		mwindow.appendChild(QuickMag.magbox);
	},

	toggleMag: function (e) {
		if (e.detail != 3)
			return;

		if (QuickMag.magOn)
			QuickMag.disableMag();
		else {
			var half_width = QuickMag.boxWidth/2;
			var boxObj = e.currentTarget.boxObject;
			var mwindow = document.getElementById("main-window");
			QuickMag.magbox.addEventListener("click", QuickMag.disableMag, false);
			QuickMag.magbox.addEventListener("DOMMouseScroll", QuickMag.adjustZoom, false);
			QuickMag.magbox.style.left = (boxObj.x + e.clientX - half_width) + "px";
			QuickMag.magbox.style.top = (boxObj.y + e.clientY - half_width) + "px";
			QuickMag.magbox.hidden = false;
			QuickMag.ignore_event = true;
			window.addEventListener("mousemove", QuickMag.trackMouse, false);
			QuickMag.calcMag(boxObj.x + e.clientX, boxObj.y + e.clientY);
			QuickMag.renderMag();
			QuickMag.magOn = !QuickMag.magOn;
		}
	},

	onLoad: function () {
		Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranchInternal).addObserver("reveal.", QuickMag, false);

		this.prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("reveal.");

		this.boxWidth = this.prefs.getIntPref("quickmag_size");

		this.canvas = document.getElementById("quickmag-canvas");
		this.canvas.width = this.canvas.height = this.boxWidth;
		this.scaled_width = Math.ceil(this.boxWidth/this.mag_factor) + 1;

		var ctx = this.canvas.getContext("2d");
		ctx.save();
		ctx.scale(this.mag_factor, this.mag_factor);

		this.magbox = document.getElementById("quickmag-box");
		this.magbox.style.left = this.magbox.style.top = "0px";
		if (getBoolPref("reveal.quickmag"))
			getBrowser().mPanelContainer.addEventListener("click", this.toggleMag, false);
	},

	onUnload: function () {
		if (getBoolPref("reveal.quickmag"))
			getBrowser().mPanelContainer.removeEventListener("click", this.toggleMag, false);
	}
};

window.addEventListener("load", function (e) { QuickMag.onLoad(); }, false);
window.addEventListener("unload", function (e) { QuickMag.onUnload(); }, false);
